@@ -27,14 +27,11 @@
 #include <lcms2.h>
 #include <math.h>
 
+#include <colord/colord.h>
+
 #include "cd-debug.h"
 #include "cd-state.h"
 #include "cd-lcms-helpers.h"
-#include "cd-client-sync.h"
-#include "cd-device-sync.h"
-#include "cd-profile-sync.h"
-#include "cd-it8.h"
-#include "cd-sensor-sync.h"
 #include "cd-session.h"
 
 typedef struct {
@@ -52,6 +49,7 @@ typedef struct {
 	CdSessionInteraction	 interaction_code_last;
 	CdSensor		*sensor;
 	CdDevice		*device;
+	CdProfile		*profile;
 	CdSensorCap		 device_kind;
 	GPtrArray		*array;
 	cmsCIEXYZ		 whitepoint;
@@ -196,7 +194,6 @@ cd_main_emit_interaction_required (CdMainPrivate *priv,
 {
 	const gchar *image = NULL;
 	const gchar *message = NULL;
-	gchar *path;
 
 	/* save so we know what was asked for */
 	priv->interaction_code_last = code;
@@ -225,17 +222,8 @@ cd_main_emit_interaction_required (CdMainPrivate *priv,
 		message = "";
 		break;
 	}
-	if (image != NULL) {
-		path = g_build_filename (DATADIR,
-					 "colord",
-					 "icons",
-					 image,
-					 NULL);
-	} else {
-		path = g_strdup ("");
-	}
 	g_debug ("CdMain: Emitting InteractionRequired(%i,%s,%s)",
-		 code, message, path);
+		 code, message, image);
 	g_dbus_connection_emit_signal (priv->connection,
 				       NULL,
 				       CD_SESSION_DBUS_PATH,
@@ -244,9 +232,8 @@ cd_main_emit_interaction_required (CdMainPrivate *priv,
 				       g_variant_new ("(uss)",
 						      code,
 						      message,
-						      path),
+						      image != NULL ? image : ""),
 				       NULL);
-	g_free (path);
 }
 
 /**
@@ -293,17 +280,38 @@ cd_main_emit_finished (CdMainPrivate *priv,
 		       CdSessionError exit_code,
 		       const gchar *message)
 {
+	GVariantBuilder builder;
+
 	/* emit signal */
 	g_debug ("CdMain: Emitting Finished(%u,%s)",
 		 exit_code, message);
+
+	/* build the dict */
+	g_variant_builder_init (&builder, G_VARIANT_TYPE_ARRAY);
+	if (exit_code == CD_SESSION_ERROR_NONE) {
+		g_variant_builder_add (&builder,
+				       "{sv}",
+				       "ProfileId",
+				       g_variant_new_string (cd_profile_get_id (priv->profile)));
+		g_variant_builder_add (&builder,
+				       "{sv}",
+				       "ProfilePath",
+				       g_variant_new_string (cd_profile_get_object_path (priv->profile)));
+	} else {
+		g_variant_builder_add (&builder,
+				       "{sv}",
+				       "ErrorDetails",
+				       g_variant_new_string (message));
+	}
+
 	g_dbus_connection_emit_signal (priv->connection,
 				       NULL,
 				       CD_SESSION_DBUS_PATH,
 				       CD_SESSION_DBUS_INTERFACE_DISPLAY,
 				       "Finished",
-				       g_variant_new ("(us)",
+				       g_variant_new ("(ua{sv})",
 						      exit_code,
-						      message != NULL ? message : ""),
+						      &builder),
 				       NULL);
 }
 
@@ -947,7 +955,6 @@ out:
 static gboolean
 cd_main_import_profile (CdMainPrivate *priv, GError **error)
 {
-	CdProfile *profile;
 	gboolean ret = TRUE;
 	gchar *filename;
 	gchar *path;
@@ -959,44 +966,42 @@ cd_main_import_profile (CdMainPrivate *priv, GError **error)
 				 NULL);
 	g_debug ("trying to import %s", path);
 	file = g_file_new_for_path (path);
-	profile = cd_client_import_profile_sync (priv->client,
-						 file,
-						 priv->cancellable,
-						 error);
-	if (profile == NULL) {
+	priv->profile = cd_client_import_profile_sync (priv->client,
+						       file,
+						       priv->cancellable,
+						       error);
+	if (priv->profile == NULL) {
 		ret = FALSE;
 		goto out;
 	}
-	g_debug ("imported %s", cd_profile_get_object_path (profile));
+	g_debug ("imported %s", cd_profile_get_object_path (priv->profile));
 
 	/* add profile to device and set default */
-	ret = cd_profile_connect_sync (profile,
+	ret = cd_profile_connect_sync (priv->profile,
 				       priv->cancellable,
 				       error);
 	if (!ret)
 		goto out;
 	ret = cd_device_add_profile_sync (priv->device,
 					  CD_DEVICE_RELATION_HARD,
-					  profile,
+					  priv->profile,
 					  priv->cancellable,
 					  error);
 	if (!ret)
 		goto out;
 	ret = cd_device_make_profile_default_sync (priv->device,
-						   profile,
+						   priv->profile,
 						   priv->cancellable,
 						   error);
 	if (!ret)
 		goto out;
 	g_debug ("set %s default on %s",
-		 cd_profile_get_id (profile),
+		 cd_profile_get_id (priv->profile),
 		 cd_device_get_id (priv->device));
 out:
 	g_free (filename);
 	g_free (path);
 	g_object_unref (file);
-	if (profile != NULL)
-		g_object_unref (profile);
 	return ret;
 }
 
@@ -2160,6 +2165,8 @@ out:
 		g_object_unref (priv->sensor);
 	if (priv->device != NULL)
 		g_object_unref (priv->device);
+	if (priv->profile != NULL)
+		g_object_unref (priv->profile);
 	if (priv->cancellable != NULL)
 		g_object_unref (priv->cancellable);
 	if (priv->it8_cal != NULL)
