@@ -31,6 +31,7 @@
 
 #include "cd-common.h"
 #include "cd-profile.h"
+#include "cd-resources.h"
 
 static void     cd_profile_finalize	(GObject     *object);
 
@@ -63,6 +64,7 @@ struct _CdProfilePrivate
 	guint				 owner;
 	gchar				**warnings;
 	GMappedFile			*mapped_file;
+	guint				 score;
 };
 
 enum {
@@ -150,6 +152,9 @@ cd_profile_set_is_system_wide (CdProfile *profile, gboolean is_system_wide)
 {
 	g_return_if_fail (CD_IS_PROFILE (profile));
 	profile->priv->is_system_wide = is_system_wide;
+
+	/* by default, prefer systemwide profiles over user profiles */
+	profile->priv->score += 1;
 }
 
 /**
@@ -215,17 +220,82 @@ cd_profile_set_object_path (CdProfile *profile)
 	g_free (path_tmp);
 }
 
+/**
+ * cd_profile_set_metadata:
+ **/
+static void
+cd_profile_set_metadata (CdProfile *profile,
+			 const gchar *property,
+			 const gchar *value)
+{
+	g_hash_table_insert (profile->priv->metadata,
+			     g_strdup (property),
+			     g_strdup (value));
+}
+
 /**
  * cd_profile_set_id:
  **/
 void
 cd_profile_set_id (CdProfile *profile, const gchar *id)
 {
+	CdStandardSpace standard_space = CD_STANDARD_SPACE_UNKNOWN;
+
 	g_return_if_fail (CD_IS_PROFILE (profile));
 
 	g_free (profile->priv->id);
 	profile->priv->id = g_strdup (id);
 
+	/* all profiles have a score initially */
+	profile->priv->score = 1;
+
+	/* http://www.color.org/srgbprofiles.xalter */
+	if (g_strcmp0 (id, "icc-34562abf994ccd066d2c5721d0d68c5d") == 0) {
+		/* sRGB_v4_ICC_preference */
+		standard_space = CD_STANDARD_SPACE_SRGB;
+		profile->priv->score = 10;
+	}
+	if (g_strcmp0 (id, "icc-fc66337837e2886bfd72e9838228f1b8") == 0) {
+		/* sRGB_v4_ICC_preference_displayclass */
+		standard_space = CD_STANDARD_SPACE_SRGB;
+		profile->priv->score = 8;
+	}
+	if (g_strcmp0 (id, "icc-29f83ddeaff255ae7842fae4ca83390d") == 0) {
+		/* sRGB_IEC61966-2-1_black_scaled */
+		standard_space = CD_STANDARD_SPACE_SRGB;
+		profile->priv->score = 6;
+	}
+	if (g_strcmp0 (id, "icc-c95bd637e95d8a3b0df38f99c1320389") == 0) {
+		/* sRGB_IEC61966-2-1_no_black_scaling */
+		standard_space = CD_STANDARD_SPACE_SRGB;
+		profile->priv->score = 4;
+	}
+
+	/* from http://download.adobe.com/pub/adobe/iccprofiles/ */
+	if (g_strcmp0 (id, "icc-dea88382d899d5f6e573b432473ae138") == 0) {
+		/* AdobeRGB1998 */
+		standard_space = CD_STANDARD_SPACE_ADOBE_RGB;
+		profile->priv->score = 10;
+	}
+	if (g_strcmp0 (id, "icc-91cf26c58e07eda724fdbf3eadce4505") == 0) {
+		/* ColorMatchRGB */
+		standard_space = CD_STANDARD_SPACE_LAST;
+	}
+	if (g_strcmp0 (id, "icc-2e54d10b392cac47226469ba2ea95bd8") == 0) {
+		/* AppleRGB */
+		standard_space = CD_STANDARD_SPACE_LAST;
+	}
+
+	/* add additional metadata to fix the GUIs */
+	if (standard_space != CD_STANDARD_SPACE_UNKNOWN) {
+		cd_profile_set_metadata (profile,
+					 CD_PROFILE_METADATA_STANDARD_SPACE,
+					 cd_standard_space_to_string (standard_space));
+		cd_profile_set_metadata (profile,
+					 CD_PROFILE_METADATA_DATA_SOURCE,
+					 CD_PROFILE_METADATA_DATA_SOURCE_STANDARD);
+	}
+
 	/* now calculate this again */
 	cd_profile_set_object_path (profile);
 }
@@ -489,9 +559,7 @@ cd_profile_set_property_internal (CdProfile *profile,
 						       g_variant_new_string (value));
 	} else {
 		/* add to metadata */
-		g_hash_table_insert (profile->priv->metadata,
-				     g_strdup (property),
-				     g_strdup (value));
+		cd_profile_set_metadata (profile, property, value);
 		cd_profile_dbus_emit_property_changed (profile,
 						       CD_PROFILE_PROPERTY_METADATA,
 						       cd_profile_get_metadata_as_variant (profile));
@@ -1382,13 +1450,14 @@ cd_profile_set_filename (CdProfile *profile,
 			 const gchar *filename,
 			 GError **error)
 {
-	cmsHPROFILE lcms_profile = NULL;
-	gboolean ret = FALSE;
-	gchar *fake_md5 = NULL;
-	gchar *data = NULL;
-	gsize len;
-	const gchar *tmp;
 	CdProfilePrivate *priv = profile->priv;
+	cmsHPROFILE lcms_profile = NULL;
+	const gchar *tmp;
+	gboolean ret = FALSE;
+	GBytes *gdata = NULL;
+	gchar *data = NULL;
+	gchar *fake_md5 = NULL;
+	gsize len;
 
 	g_return_val_if_fail (CD_IS_PROFILE (profile), FALSE);
 
@@ -1432,15 +1501,33 @@ cd_profile_set_filename (CdProfile *profile,
 #endif
 	}
 
-	/* parse the ICC file */
-	lcms_profile = cmsOpenProfileFromFile (filename, "r");
-	if (lcms_profile == NULL) {
-		g_set_error (error,
-			     CD_PROFILE_ERROR,
-			     CD_PROFILE_ERROR_FAILED_TO_PARSE,
-			     "failed to parse %s",
-			     filename);
-		goto out;
+	/* find out if we have a GResource copy */
+	if (g_str_has_prefix (filename, "/usr/share/color/icc/colord/")) {
+		data = g_build_filename ("/org/freedesktop/colord",
+					 "profiles",
+					 filename + 28,
+					 NULL);
+		gdata = g_resource_lookup_data (cd_get_resource (),
+						data,
+						G_RESOURCE_LOOKUP_FLAGS_NONE,
+						NULL);
+	}
+
+	/* load the ICC file using lcms */
+	if (gdata != NULL) {
+		g_debug ("Using built-in %s", data);
+		lcms_profile = cmsOpenProfileFromMem (g_bytes_get_data (gdata, NULL),
+						      g_bytes_get_size (gdata));
+	} else {
+		lcms_profile = cmsOpenProfileFromFile (filename, "r");
+		if (lcms_profile == NULL) {
+			g_set_error (error,
+				     CD_PROFILE_ERROR,
+				     CD_PROFILE_ERROR_FAILED_TO_PARSE,
+				     "failed to parse %s",
+				     filename);
+			goto out;
+		}
 	}
 
 	/* set the virtual profile from the lcms profile */
@@ -1478,6 +1565,8 @@ cd_profile_set_filename (CdProfile *profile,
 out:
 	g_free (data);
 	g_free (fake_md5);
+	if (gdata != NULL)
+		g_bytes_unref (gdata);
 	if (lcms_profile != NULL)
 		cmsCloseProfile (lcms_profile);
 	return ret;
@@ -1617,6 +1706,22 @@ cd_profile_get_metadata_item (CdProfile *profile, const gchar *key)
 	return g_hash_table_lookup (profile->priv->metadata, key);
 }
 
+/**
+ * cd_profile_get_score:
+ *
+ * Profiles from official vendors such as http://www.color.org/ should be
+ * more important than profiles generated from source data.
+ *
+ * Return value: A number which corresponds to the importance of the profile,
+ * where larger numbers have more importance than lower numbers.
+ **/
+guint
+cd_profile_get_score (CdProfile *profile)
+{
+	g_return_val_if_fail (CD_IS_PROFILE (profile), 0);
+	return profile->priv->score;
+}
+
 /**
  * cd_profile_get_kind:
  **/
