@@ -26,6 +26,8 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <sqlite3.h>
+#include <glib/gstdio.h>
 
 #include "cd-common.h"
 #include "cd-device-array.h"
@@ -45,6 +47,11 @@ colord_profile_func (void)
 
 	cd_profile_set_id (profile, "dave");
 	g_assert_cmpstr (cd_profile_get_id (profile), ==, "dave");
+	g_assert_cmpint (cd_profile_get_score (profile), ==, 1);
+
+	/* system-wide profiles have a larger importance */
+	cd_profile_set_is_system_wide (profile, TRUE);
+	g_assert_cmpint (cd_profile_get_score (profile), ==, 2);
 
 	g_object_unref (profile);
 }
@@ -90,7 +97,7 @@ colord_device_func (void)
 
 	/* add profile again */
 	ret = cd_device_add_profile (device,
-				     CD_DEVICE_RELATION_HARD,
+				     CD_DEVICE_RELATION_SOFT,
 				     cd_profile_get_object_path (profile),
 				     0,
 				     &error);
@@ -98,6 +105,15 @@ colord_device_func (void)
 	g_assert (!ret);
 	g_clear_error (&error);
 
+	/* add profile again */
+	ret = cd_device_add_profile (device,
+				     CD_DEVICE_RELATION_HARD,
+				     cd_profile_get_object_path (profile),
+				     0,
+				     &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
 	/* add profile that does not exist */
 	ret = cd_device_add_profile (device,
 				     CD_DEVICE_RELATION_SOFT,
@@ -159,19 +175,118 @@ colord_device_array_func (void)
 }
 
 static void
-cd_mapping_db_func (void)
+cd_mapping_db_alter_func (void)
 {
 	CdMappingDb *mdb;
-	GError *error = NULL;
+	const gchar *db_filename = "/tmp/mapping.db";
+	const gchar *statement;
 	gboolean ret;
-	GPtrArray *array;
+	GError *error = NULL;
+	gint rc;
+	sqlite3 *db;
 
 	/* create */
 	mdb = cd_mapping_db_new ();
 	g_assert (mdb != NULL);
 
-	/* connect, which should create it for us */
-	ret = cd_mapping_db_load (mdb, "/tmp/mapping.db", &error);
+	/* setup v0.1.0 database for altering */
+	g_unlink (db_filename);
+	rc = sqlite3_open (db_filename, &db);
+	g_assert_cmpint (rc, ==, SQLITE_OK);
+	statement = "CREATE TABLE mappings ("
+		    "device TEXT,"
+		    "profile TEXT);";
+	sqlite3_exec (db, statement, NULL, NULL, NULL);
+	statement = "INSERT INTO mappings (device, profile) VALUES ('dev1', 'prof1')";
+	sqlite3_exec (db, statement, NULL, NULL, NULL);
+
+	/* connect */
+	ret = cd_mapping_db_load (mdb, db_filename, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* check we ALTERed it correctly */
+	statement = "SELECT timestamp FROM mappings LIMIT 1";
+	rc = sqlite3_exec (db, statement, NULL, NULL, NULL);
+	g_assert_cmpint (rc, ==, SQLITE_OK);
+
+	sqlite3_close (db);
+	g_object_unref (mdb);
+}
+
+/**
+ * cd_mapping_db_test_cb:
+ **/
+static gint
+cd_mapping_db_test_cb (void *data, gint argc, gchar **argv, gchar **col_name)
+{
+	gchar **tmp = (gchar **) data;
+	*tmp = g_strdup (argv[0]);
+	return 0;
+}
+
+static void
+cd_mapping_db_convert_func (void)
+{
+	CdMappingDb *mdb;
+	const gchar *db_filename = "/tmp/mapping.db";
+	const gchar *statement;
+	gboolean ret;
+	gchar *device_id = NULL;
+	GError *error = NULL;
+	gint rc;
+	sqlite3 *db;
+
+	/* create */
+	mdb = cd_mapping_db_new ();
+	g_assert (mdb != NULL);
+
+	/* setup v0.1.8 database for converting */
+	g_unlink (db_filename);
+	rc = sqlite3_open (db_filename, &db);
+	g_assert_cmpint (rc, ==, SQLITE_OK);
+	statement = "CREATE TABLE mappings ("
+		    "device TEXT,"
+		    "profile TEXT,"
+		    "timestamp INTEGER DEFAULT 0);";
+	sqlite3_exec (db, statement, NULL, NULL, NULL);
+	statement = "INSERT INTO mappings (device, profile, timestamp) VALUES ('dev1', 'prof1', 12345)";
+	sqlite3_exec (db, statement, NULL, NULL, NULL);
+
+	/* connect, which should convert it for us */
+	ret = cd_mapping_db_load (mdb, db_filename, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* check we converted it correctly */
+	statement = "SELECT timestamp FROM mappings_v2 LIMIT 1";
+	rc = sqlite3_exec (db, statement,
+			   cd_mapping_db_test_cb, &device_id, NULL);
+	g_assert_cmpint (rc, ==, SQLITE_OK);
+	g_assert_cmpstr (device_id, ==, "12345");
+	g_free (device_id);
+
+	sqlite3_close (db);
+	g_object_unref (mdb);
+}
+
+static void
+cd_mapping_db_func (void)
+{
+	CdMappingDb *mdb;
+	const gchar *db_filename = "/tmp/mapping.db";
+	gboolean ret;
+	GError *error = NULL;
+	GPtrArray *array;
+	guint64 timestamp;
+
+	/* create */
+	mdb = cd_mapping_db_new ();
+	g_assert (mdb != NULL);
+
+	/* connect, which should create a v2 table for us */
+	g_unlink (db_filename);
+	ret = cd_mapping_db_load (mdb, db_filename, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
 
@@ -192,10 +307,19 @@ cd_mapping_db_func (void)
 	g_assert (ret);
 
 	/* remove one */
-	ret = cd_mapping_db_remove (mdb, "device1", "profile2", &error);
+	ret = cd_mapping_db_clear_timestamp (mdb, "device1", "profile2", &error);
 	g_assert_no_error (error);
 	g_assert (ret);
 
+	/* ensure timestamp really is zero */
+	timestamp = cd_mapping_db_get_timestamp (mdb,
+						 "device1",
+						 "profile2",
+						 &error);
+	g_assert_no_error (error);
+	g_assert (timestamp != G_MAXUINT64);
+	g_assert_cmpint (timestamp, ==, 0);
+
 	/* get the profiles for a device */
 	array = cd_mapping_db_get_profiles (mdb, "device1", &error);
 	g_assert_no_error (error);
@@ -337,6 +461,8 @@ main (int argc, char **argv)
 	g_log_set_fatal_mask (NULL, G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
 
 	/* tests go here */
+	g_test_add_func ("/colord/mapping-db{alter}", cd_mapping_db_alter_func);
+	g_test_add_func ("/colord/mapping-db{convert}", cd_mapping_db_convert_func);
 	g_test_add_func ("/colord/mapping-db", cd_mapping_db_func);
 	g_test_add_func ("/colord/device-db", cd_device_db_func);
 	g_test_add_func ("/colord/profile", colord_profile_func);
